from math import sqrt

import numpy as np

from knn import distance_matrix, evaluate_knn, knn_predict


def test_distance_matrix():
    matrix_a = np.array([[1, 2], [3, 4], [5, 6]])
    matrix_b = np.array([[10, 9], [8, 7]])

    expected = np.array(
        [
            [sqrt((1 - 10) ** 2 + (2 - 9) ** 2), sqrt((1 - 8) ** 2 + (2 - 7) ** 2)],
            [sqrt((3 - 10) ** 2 + (4 - 9) ** 2), sqrt((3 - 8) ** 2 + (4 - 7) ** 2)],
            [sqrt((5 - 10) ** 2 + (6 - 9) ** 2), sqrt((5 - 8) ** 2 + (6 - 7) ** 2)],
        ]
    )

    result = distance_matrix(matrix_a, matrix_b)

    print(result)
    print(expected)

    assert np.allclose(result, expected)


def test_knn_predict():
    dists = np.array(
        [
            [1, 2, 3, 4, 5, 6],
            [6, 5, 4, 3, 2, 1],
        ]
    )
    labels_train = np.array([1, 1, 1, 2, 2, 2])
    k = 3

    expected = np.array([1, 2])

    result = knn_predict(dists, labels_train, k)

    assert np.allclose(result, expected)


def test_evaluate_knn():
    data_train = np.array(
        [
            [10, 10],
            [20, 20],
            [30, 30],
            [40, 40],
            [50, 50],
        ]
    )
    labels_train = np.array([5, 4, 3, 2, 1])
    data_test = np.array(
        [
            [19, 19],
            [42, 42],
        ]
    )
    labels_test = np.array([4, 2])
    k = 1

    expected = 1.0

    result = evaluate_knn(data_train, labels_train, data_test, labels_test, k)

    assert np.allclose(result, expected)
