import numpy as np

from read_cifar import read_cifar, read_cifar_batch, split_dataset

DATASET_PATH = "./data/cifar-10-batches-py"


def test_read_cifar_batch():
    data, labels = read_cifar_batch(f"{DATASET_PATH}/data_batch_1")

    # check dimensions
    assert data.shape == (10000, 3072)
    assert labels.shape == (10000,)
    # check types
    assert data.dtype == np.float32
    assert labels.dtype == np.int64


def test_read_cifar():
    data, labels = read_cifar(DATASET_PATH)

    # check dimensions
    assert data.shape == (60000, 3072)
    assert labels.shape == (60000,)
    # check types
    assert data.dtype == np.float32
    assert labels.dtype == np.int64


def test_split_dataset():
    cifar_data, cifar_labels = read_cifar(DATASET_PATH)
    split = 0.8

    datapoint_size = cifar_data.shape[1]

    data_train, labels_train, data_test, labels_test = split_dataset(
        cifar_data, cifar_labels, split
    )

    expected_train_size = round(split * cifar_data.shape[0])
    expected_test_size = cifar_data.shape[0] - expected_train_size

    # check dimensions
    assert data_train.shape == (expected_train_size, datapoint_size)
    assert labels_train.shape == (expected_train_size,)
    assert data_test.shape == (expected_test_size, datapoint_size)
    assert labels_test.shape == (expected_test_size,)

    # check types
    assert data_train.dtype == cifar_data.dtype
    assert labels_train.dtype == cifar_labels.dtype
    assert data_test.dtype == cifar_data.dtype
    assert labels_test.dtype == cifar_labels.dtype

    # check that the first data and label are still in the dataset and match indices
    # pick random index in the training set
    i = np.random.randint(0, expected_train_size)
    random_data = data_train[i]
    random_label = labels_train[i]
    # check that the data and label match in original dataset
    original_index = np.where(np.all(cifar_data == random_data, axis=1))[0][0]
    assert cifar_labels[original_index] == random_label
